import mx.data.components.datasetclasses.DataSetError;

class com.macromedia.timeentry.TimeEntryData extends mx.screens.Form {
	
	//var wscLoadTimeEntry:mx.data.components.WebServiceConnector;
	var conLoadTimeEntry: mx.data.components.XMLConnector;
		
		
	//var wscSaveTimeEntry:mx.data.components.WebServiceConnector;
	var conSaveTimeEntry:mx.data.components.WebServiceConnector;	
	
	//var wscProject:mx.data.components.WebServiceConnector;
	var conProject:mx.data.components.XMLConnector;	
	
	var rdbmsTimeEntry:mx.data.components.RDBMSResolver;
	var dsTimeEntry:mx.data.components.DataSet;
	var dsProject:mx.data.components.DataSet;
	var frmLog:com.macromedia.timeentry.Log;
	
	/**
	  initialize class.
	*/
	function TimeEntryData() {
		visible = false;
		addEventListener("load", onLoad);
		//create pointer to log form
		frmLog = _root.application.frmLog;
	}
	
	/**
	  call on the DataSet to append a new time entry to its collection of data items.
	*/
	function addTimeEntry() {
		dsTimeEntry.addItem();
	}
	
	/**
	  fires when a new entry is added to the DataSet.  Set the current date.  Note: this same behavior can be accomplished
	  by setting the default value to "now" in the Schema tab for the DataSet.
	*/
	function dsTimeEntryAddItemHandler(evt:Object):Void {
		evt.item.DATE = new Date();
		dsTimeEntryIteratorScrolledHandler();
	}

	/**
	  fires when errors are received from the server in response to an attempted update.  Errors are passed to the Reconcile
	  dialog and displayed to the user.
	*/
	function dsTimeEntryResolveDeltaHandler(evt:Object):Void {
		var deltaData = evt.data;
		var ds = evt.target;
		var wndReconcile = mx.managers.PopUpManager.createPopUp(_root, mx.containers.Window, false, { title:"Reconcile Dialog", contentPath:"frmReconcile", closeButton:true});
		//complete event is undocumented.  It fires when the content for a Window has been loaded.
		wndReconcile.addEventListener("complete", function(evt: Object) { evt.target.content.setReconcileData(ds, deltaData); } );
		wndReconcile.addEventListener("click", function(evt: Object) { evt.target.content.reconcileChanges(); evt.target.deletePopUp(); } );
		wndReconcile.setSize(410,300);
	}
	
	/**
	  create an index on the Project DataSet after it has loaded data.  The index is needed for when we call the find function
	  to sync to the correct lookup record whenever a new record is selected in the Time Entry DataSet (Part of the dynamic 
	  lookup functionality).																										
	*/
	function dsProjectAfterLoadedHandler():Void {
		dsProject.addSort("projId", ["data"] );
		//call loadTimeEntryData here guarantees that the Project DataSet is loaded and the index is set before the TimeEntry
		//DataSet can be loaded. Once the TimeEntry DataSet is loaded it makes a call to sync up the lookup record.  This call
		//requires that the Project DataSte has an index.
		loadTimeEntryData();
	}

	/**
	  force the TimeEntry DataSet to scroll so that the IteratorScrolled handler fires.  Make sure that the Project DataSet
	  stays in sync with the TimeEntry DataSet (part of the dynamic lookup functionality).
	*/
	function dsTimeEntryAfterLoadedHandler():Void {
		dsTimeEntryIteratorScrolledHandler();
	}

	/**
	  recalculate cost each time a field value is changed.
	*/
	function dsTimeEntryCalcFieldsHandler():Void {
		dsTimeEntry.COST = (dsTimeEntry.DURATION * dsTimeEntry.RATE);
	}
	
	/**
	  fires when a new record is selected within the TimeEntry DataSet.  Find a record within the Project DataSet that matches
	  the PROJID fields in the Time Entry DataSet (part of the dynamic lookup functionality).
	*/
	function dsTimeEntryIteratorScrolledHandler(): Void {
		this.findProjectByID(dsTimeEntry.PROJID);
	}

	/**
	  filter records based on date range.  If a null is passed in as a parameter the filter is turned off.
	*/
	function filterTimeEntriesByDate( dtFrom:mx.controls.DateField, dtTo:mx.controls.DateField ) {
		dsTimeEntry.filtered = false;
		dsTimeEntry.filterFunc = function(item:Object) {
		  // filter out records that are not within the date range
		  return( (item.DATE >= dtFrom) && (item.DATE <= dtTo) );
		}
		dsTimeEntry.filtered = ( (dtFrom != null) && (dtTo != null) );
	}

	/**
	  find a project record by its ID.  If the record is found it is made the current record for the Project DataSet.
	*/
	function findProjectByID(id:Number) {
		dsProject.find([id]);
	}
	
	/**
	  load data for the application. The afterLoaded event for the Project DataSet takes care of making the call to load 
	  the TimeEntry data.
	*/
	function loadData():Void {
		loadProjectData();
	}
	
	/**
	  retrieve Project data from the server or the local machine depending on user settings. 
	*/
	function loadProjectData():Void {
		if ( _root.application.getDataOffline() ) {
			try {
				dsProject.loadFromSharedObj("TimeEntry");
			}
			catch(e:DataSetError) {
				trace("Unable to load shared object."); 
			}
		} 
		else {
			// PhilFlash
			//wscProject.trigger();
			conProject.trigger();
		}
	}
	
	/**
	  retrieve Time Entry data from the server or the local machine depending on user settings. This call is made from the
	  afterLoaded event of the project DataSet. This is done to ensure that an index can be placed on the Project DataSet
	  before the TimeEntry DataSet is loaded.
	*/
	function loadTimeEntryData():Void {
		if ( _root.application.getDataOffline() ) {
			try {
				dsTimeEntry.loadFromSharedObj("TimeEntry");
			}
			catch(e:DataSetError) {
				trace("Unable to load shared object."); 
			}
		} 
		else {
			// PhilFlash
			//wscLoadTimeEntry.trigger();
			conLoadTimeEntry.trigger();
		}
	}

	/**
	  initialize components that live within this form.  Setup listeners for events that we are interested in 
	  responding to within the context of the TimeEntryData class.
	*/
	function onLoad():Void {
		rdbmsTimeEntry.addEventListener("beforeApplyUpdates", function(evt: Object) {evt.target._parent.rdbmsTimeEntryBeforeApplyUpdatesHandler(evt); } );
		rdbmsTimeEntry.addEventListener("reconcileResults", function(evt: Object) {evt.target._parent.rdbmsTimeEntryReconcileResultsHandler(evt); } );
		dsProject.addEventListener("afterLoaded", function(evt: Object) {evt.target._parent.dsProjectAfterLoadedHandler(); } );
		dsTimeEntry.addEventListener("afterLoaded", function(evt: Object) {evt.target._parent.dsTimeEntryAfterLoadedHandler(); } );
		dsTimeEntry.addEventListener("iteratorScrolled", function(evt: Object) {evt.target._parent.dsTimeEntryIteratorScrolledHandler(); } );
		dsTimeEntry.addEventListener("addItem", function(evt: Object) {evt.target._parent.dsTimeEntryAddItemHandler(evt); } );
		dsTimeEntry.addEventListener("calcFields", function(evt: Object) {evt.target._parent.dsTimeEntryCalcFieldsHandler(); } );
		dsTimeEntry.addEventListener("resolveDelta", function(evt: Object) {evt.target._parent.dsTimeEntryResolveDeltaHandler(evt); } );
	}
	
	/**
	  modify the updatePacket so that date values are formatted correctly for Access (target database).  Also, add field
	  generation information to the header of the updatePacket.  The CFC that reads the updatePacket looks for this
	  information. If a field is identified as auto-generated the CFC will pass a value back from the database.  We use this
	  functionality so that the database can create and send back primary key values for new records.
	*/
	function rdbmsTimeEntryBeforeApplyUpdatesHandler(evt:Object):Void {
		// PhilFlash : Not necessary for Access/SQL-Server with ADO and ISO Date Format
		
		////format dates for Access database
		//var dateNodes = mx.xpath.XPathAPI.selectNodeList( evt.updatePacket.firstChild, "update_packet/*/field[@type='Date']" );
		//for (var i=0; i<dateNodes.length; i++) {
		//	//place [] around fieldname because date is a keyword
		//	dateNodes[i].attributes.name = "[" + 
		// 	dateNodes[i].attributes.name + "]";
		//	//place ## around old value of date field
		//	if (dateNodes[i].attributes.oldValue != undefined) {
		//		dateNodes[i].attributes.oldValue = "#" + dateNodes[i].attributes.oldValue + "#";
		//	} //if
		//	//place ## around new value of date field
		//		if (dateNodes[i].attributes.newValue != undefined) {
		//			dateNodes[i].attributes.newValue = "#" + dateNodes[i].attributes.newValue + "#";
		//	} //if
		//} //for
		
		// PhilFlash : ADD for errors
		// Generates Errors for SQL
		if (rootForm.frmTimeEntry.generateError) {
		
			// for Update operation, generate a fieldname error
			var nodes = mx.xpath.XPathAPI.selectNodeList( evt.updatePacket.firstChild, "update_packet/update/field[@key='false']" );
			for (var i=0; i<nodes.length; i++) {
				//add BUG around fieldname to generate an error
				nodes[i].attributes.name = "BUG_" + nodes[i].attributes.name;
			}
			
			// for delete operation, generate an ID error
			var nodes = mx.xpath.XPathAPI.selectNodeList( evt.updatePacket.firstChild, "update_packet/delete/field[@name='ID']" );
			for (var i=0; i<nodes.length; i++) {
				//add BUG around ID to generate an error
				if (nodes[i].attributes.oldValue != undefined) {
					nodes[i].attributes.oldValue = "BUG_" + nodes[i].attributes.oldValue;
				}
			}
			
			// for insert operation, generate a fieldname error
			var nodes = mx.xpath.XPathAPI.selectNodeList( evt.updatePacket.firstChild, "update_packet/insert/field[@key='false']" );
			for (var i=0; i<nodes.length; i++) {
				//add BUG around fieldname to generate an error
				nodes[i].attributes.name = "BUG_" + nodes[i].attributes.name;
			}
		}

		//add an attribute to the update packet to tell the CFC that we want it to generate values for the id field
		var firstChild:XMLNode = evt.updatePacket.firstChild;
		firstChild.attributes.autoIncField = "ID";

		//send updatePacket to the log form
		frmLog.addTransaction();
		frmLog.addNewEntry("UPDATE PACKET - " + new Date() + "\n\n", evt.updatePacket +"\n\n");
	}

	/**
	  send resultPacket to the log form
	*/
	function rdbmsTimeEntryReconcileResultsHandler(evt:Object):Void {
		frmLog.addNewEntry("RESULT PACKET - " + new Date() + "\n\n", evt.target.updateResults +"\n\n");
	}

	/**
	  call on the DataSet to remove the selected time entry from its collection of data items.
	*/
	function removeTimeEntry() {
		dsTimeEntry.removeItem();
	}
	
	/**
	  save project data to the server or local machine depending on user settings
	*/
	function saveProjectData() {
		if ( _root.application.getDataOffline() ) {
			try {
				dsProject.saveToSharedObj("Project");
			}
			catch(e:DataSetError) {
				trace("Unable to create shared object");
			}
		} 
		else {
			dsProject.applyUpdates();
		}
	}

	/**
	  save time entry data to the server or local machine depending on user settings
	*/
	function saveTimeEntryData() {
		if ( _root.application.getDataOffline() ) {
			try {
				dsTimeEntry.saveToSharedObj("TimeEntry");
			}
			catch(e:DataSetError) {
				trace("Unable to create shared object");
			}
		} 
		else {
			dsTimeEntry.applyUpdates();
		}
	}

}